@@ -49,38 +49,8 @@ namespace bgeot
     scalar_type c_max;
 
     /// comparaison function
-    int operator()(const base_node &x, const base_node &y) const
-    { 
-      size_type s = x.size(); 
-      scalar_type c1 = c_max, c2 = c_max*10.0;
-      if (y.size() != s)
-	throw dimension_error(
-	    "imbricated_box_less::operator() : dimension error");
-
-      base_node::const_iterator itx=x.begin(), itex=x.end(), ity=y.begin();
-      for (; itx != itex; ++itx, ++ity)
-      {
-	long a = long(sfloor((*itx) * c1)), b = long(sfloor((*ity) * c1));
-	if ((dal::abs(a) > 10) || (dal::abs(b) > 10))
-	{ 
-	  (*((int *)(&exp_max)))++; *((scalar_type *)(&c_max)) /= 10.0;
-	  return (*this)(x,y);
-	}
-	if (a < b) return -1; else if (a > b) return 1;
-      }
-
-      for (int e = exp_max; e >= exp_min; --e, c1 *= 10.0, c2 *= 10.0)
-      {
-	itx = x.begin(), itex = x.end(), ity = y.begin();
-	for (; itx != itex; ++itx, ++ity)
-	{
-	  int a = int(sfloor(((*itx) * c2) - sfloor((*itx) * c1)*10.0));
-	  int b = int(sfloor(((*ity) * c2) - sfloor((*ity) * c1)*10.0));
-	  if (a < b) return -1; else if (a > b) return 1;
-	}
-      }
-      return 0;
-    }
+    int operator()(const base_node &x, const base_node &y) const;
+    
     imbricated_box_less(int emi = -15, int ema = -2)
     { exp_max = ema; exp_min = emi;  c_max = pow(10.0, -exp_max); }
   };
@@ -159,73 +129,15 @@ namespace bgeot
       }
 
       /// Add point p to the list of points.
-      void add_point(base_node p)
-      { ptab.add(p); }
+      size_type add_point(base_node p) { return ptab.add(p); }
+      
+      /// Add point p to the list of points.
+      size_type add_point_norepeat(base_node p)
+      { return ptab.add_norepeat(p); }
       
       /// Find all the points present in the box between min and max.
       void points_in_box(dal::bit_vector &pt, const base_node &min,
-			                      const base_node &max) const
-      {
-	size_type s = min.size();
-	TAB_TYPE::const_sorted_iterator it, ite;
-
-	it = ptab.sorted_ge(min); ite = ptab.sorted_ge(max);
-	base_node::const_iterator itl, itmin, itmax, itmine = min.end();
-	for(; it != ite; ++it)
-	{ 
-	  bool isin = true;
-	  itl = (*it).begin(); itmin = min.begin(); itmax = max.begin();
-	  for (; itmin != itmine; ++itmin, ++itmax, ++itl)
-	    if (*itl < *itmin || *itl > *itmax) { isin = false; break; }
-	  if (isin) pt.add(it.index());
-	}
-
-	/* The following is a version with a partition, avoiding default */
-	/* of the simple search, but which is slower .. in the mean.     */
-/* 	size_type s = min.size(), i; */
-/* 	base_node c(s),boxmin(s),boxmax(s),cbox(s), iboxmin(s), iboxmax(s); */
-/* 	TAB_TYPE::const_sorted_iterator it, ite; */
-
-/* 	// cout << "initial box : " << min << " :: " << max << endl; */
-
-/* 	for (i = 0; i < s; ++i) */
-/* 	{ */
-/* 	  c[i] = pow(10.0, */
-/* 		 rint(std::log10(std::max(EPS, max[i] - min[i])))); */
-/* 	  boxmin[i] = floor(min[i] / c[i]) * c[i]; */
-/* 	  boxmax[i] = ceil(max[i] / c[i]) * c[i]; */
-/* 	} */
-/* 	// cout << "max box : " << boxmin << " :: " << boxmax << endl; */
-/* 	// cout << "steps : " << c << endl; */
-
-/* 	cbox = boxmin; */
-/* 	while(cbox[s-1] < boxmax[s-1]-EPS) */
-/* 	{ */
-/* 	  // intersection */
-/* 	  for (i = 0; i < s; ++i) */
-/* 	  {  */
-/* 	    iboxmin[i]=std::max(cbox[i], min[i]); */
-/* 	    iboxmax[i]=std::max(std::min(cbox[i]+c[i], max[i]), iboxmin[i]); */
-/* 	  } */
-/* 	  // recherche des points entre iboxmin et iboxmax */
-/* 	  it = ptab.sorted_ge(&iboxmin); */
-/* 	  ite = ptab.sorted_ge(&iboxmax); */
-/* 	  for(; it != ite; ++it) */
-/* 	  {  */
-/* 	    bool isin = true; */
-/* 	    for (i = 0; i < s; ++i) */
-/* 	      if ((*(*it))[i] < min[i] || (*(*it))[i] > max[i]) */
-/* 		{ isin = false; break; } */
-/* 	    if (isin) pt.add(it.index()); */
-/* 	  } */
-
-/* 	  // incrementation */
-/* 	  i = 0; cbox[0] += c[0]; */
-/* 	  while((cbox[i] >= boxmax[i]-EPS) && (i < s-1)) */
-/* 	  { cbox[i] = boxmin[i]; ++i; cbox[i] += c[i]; } */
-/* 	} */
-	
-      }
+			                      const base_node &max) const;
 
       /** Search all the points in the convex cv, which is the transformation
        *  of the convex cref via the geometric transformation pgt.
