@@ -88,78 +88,9 @@ namespace bgeot
       { dimension = dim_type(-1); mesh_structure::clear(); pts.clear(); }
 
       mesh(void) { dimension = dim_type(-1); }
-
-      int write_to_file(std::ostream &ost) const;
-      int read_from_file(std::istream &ist);
  
   };
 
-  template<class PT, class PT_TAB>
-    int mesh<PT, PT_TAB>::read_from_file(std::istream &ist)
-  {
-    dal::bit_vector npt;
-    dal::dynamic_array<double> tmpv;
-    char tmp[100];
-    bool te = false, please_get = true;
-
-    clear();
-    ist.seekg(0);
-    ftool::read_untill(ist, "BEGIN POINTS LIST");
-
-    while (!te)
-    {
-      if (please_get) ftool::get_token(ist, tmp, 99); else please_get = true;
-
-      if (!strcmp(tmp, "END"))
-      { te = true; }
-      else if (!strcmp(tmp, "POINT"))
-      {
-	ftool::get_token(ist, tmp, 99);
-        size_type ip = atoi(tmp);
-        dim_type d = 0;
-	if (npt.is_in(ip))
-	{
-	  cerr << "BGEOT : Fatal Error, two points with"
-	       << "the same index. loading aborted" << endl; return -1;
-	}
-	npt.add(ip);
-	ftool::get_token(ist, tmp, 99);
-	while (isdigit(tmp[0]) || tmp[0] == '-' || tmp[0] == '+'
-	                       || tmp[0] == '.')
-	{ tmpv[d++] = atof(tmp); ftool::get_token(ist, tmp, 99); }
-	please_get = false;
-	if (dimension == dim_type(-1)) dimension = d; else if (dimension != d)
-	{ cerr << "BGEOT : Points of different dimensions\n"; return -1; }
-	
-	PT v(d);
-	for (size_type i = 0; i < d; i++) v[i] = tmpv[i];
-	points()[ip] = v;
-      }
-      else
-      { cerr << "BGEOT : Syntax error in file\n"; return -1; }
-    }
-    return mesh_structure::read_from_file(ist);
-  }
-
-  template<class ITER> void _write_point_to_file(std::ostream &ost, ITER b, ITER e)
-  { for ( ; b != e; ++b) ost << "  " << *b; ost << endl; }
-
-  template<class PT, class PT_TAB>
-    int mesh<PT, PT_TAB>::write_to_file(std::ostream &ost) const
-  {
-    ost << endl << "BEGIN POINTS LIST" << endl << endl;
-    mesh_point_st_ct::const_iterator b = point_structures().begin();
-    mesh_point_st_ct::const_iterator e = point_structures().end();
-    for (size_type i = 0; b != e; ++b, ++i)
-      if ( (*b).is_valid() )
-      {
-	ost << "  POINT  " << i;
-	_write_point_to_file(ost, points()[i].begin(), points()[i].end());
-      }
-    ost << endl << "END POINTS LIST" << endl << endl;
-    return mesh_structure::write_to_file(ost);
-  }
-
 }  /* end of namespace bgeot.                                              */
 
 
