@@ -185,6 +185,7 @@ namespace bgeot
       bool point_is_valid(size_type i)
       { return (points_tab[i].first != size_type(-1)); }
       ref_mesh_point_ind_ct ind_points_of_convex(size_type ic) const;
+      size_type local_ind_of_convex_point(size_type ic, size_type ip) const;
       size_type ind_dir_point_of_convex(size_type ic, size_type j) const
       {
 	return ind_points_of_convex(ic)
@@ -197,9 +198,13 @@ namespace bgeot
       { return convex_tab[i].cstruct->nb_points(); }
       void swap_convex(size_type i, size_type j);
 
+
+      template<class ITER>
+       size_type add_convex_noverif(pconvex_structure cs, ITER ipts,
+				    size_type to_index = size_type(-1));
       template<class ITER>
 	size_type add_convex(pconvex_structure cs,
-			     ITER ipts, bool *present = NULL);
+			     ITER ipts, bool *present = 0);
       template<class ITER> size_type add_simplex(dim_type dim, ITER ipts)
       { return add_convex(simplex_structure(dim), ipts); }
       size_type add_segment(size_type a, size_type b);
@@ -232,8 +237,6 @@ namespace bgeot
       void optimize_structure(void);
       void clear(void);
       void stat(void);
-      int write_to_file(std::ostream &ost) const;
-      int read_from_file(std::istream &ist);
      
   };
 
@@ -337,25 +340,22 @@ namespace bgeot
     return true;
   }
 
-
   template<class ITER>
-    size_type mesh_structure::add_convex(pconvex_structure cs,
-					 ITER ipts, bool *present)
+    size_type mesh_structure::add_convex_noverif(pconvex_structure cs,
+						 ITER ipts, size_type is)
   {
     short_type nb = cs->nb_points();
-    mesh_convex_with_points_ind_ct<ITER>
-      ct = convex_with_points(*this, nb, ipts);
-    size_type is = *(ct.begin());
-    if (present != NULL) *present = false;
-
-    if (!ct.empty() && structure_of_convex(is) == cs)
-    { if (present != NULL) *present = true; return is; }
-    
     mesh_convex_structure s; s.cstruct = cs; s.pts = point_links.alloc(nb);
     if (nb > 0)
       { point_lists[s.pts+nb-1] = 0; point_links[s.pts+nb-1].next = 0; }
     dal::copy_n(ipts, nb, point_lists.begin()+s.pts);
-    is = convex_tab.add(s);
+
+    if (is != size_type(-1)) {
+      if (convex_index()[is]) sup_convex(is);
+      convex_tab.add_to_index(is, s);
+    }
+    else
+      is = convex_tab.add(s);
 
     mesh_link_ct::iterator ipl = point_links.begin(); ipl += s.pts;
     for (short_type i = 0; i < nb; ++i, ++ipts, ++ipl)
@@ -368,6 +368,20 @@ namespace bgeot
     return is;
   }
 
+  template<class ITER>
+    size_type mesh_structure::add_convex(pconvex_structure cs,
+					 ITER ipts, bool *present) {
+    if (present != 0) *present = false;
+    mesh_convex_with_points_ind_ct<ITER>
+      ct = convex_with_points(*this, cs->nb_points(), ipts);
+    typename mesh_convex_with_points_ind_ct<ITER>::const_iterator
+      it = ct.begin(), ite = ct.end();
+    for (; it != ite; ++it)
+      if (structure_of_convex(*it) == cs)
+	{ if (present != 0) *present = true; return *it; }
+    return add_convex_noverif(cs, ipts);
+  }
+
   template<class ITER>
     void mesh_structure::sup_convex_with_points(ITER ipts, short_type nb)
   {
