@@ -34,7 +34,7 @@
 #define __DAL_STD_H
 
 #define __GETFEM_VERSION 1
-#define __GETFEM_REVISION 0
+#define __GETFEM_REVISION 2
 
 #define __DAL_VERSION 1
 #define __DAL_REVISION 0
@@ -63,13 +63,21 @@
 /*	S.T.L. Headers.						          */
 /* ********************************************************************** */
 
+#include <exception>
+#include <typeinfo>
 #include <stdexcept>
 #include <iterator>
 #include <algorithm>
 #include <vector>
 #include <string>
-#include <strstream>
 
+#if (!defined(__GNUC__) || __GNUC__ < 3)
+# include <strstream>
+# define _STRINGSTREAM std::strstream
+#else
+# include <sstream>
+# define _STRINGSTREAM std::stringstream
+#endif
 using std::endl;
 using std::cout;
 using std::cerr;
@@ -332,18 +340,42 @@ typedef unsigned char uint8_type;
     cerr << "|  A bad allocation has been detected !!!  |\n";\
     cerr << "============================================\n";\
   }\
+  catch(std::bad_typeid) { \
+    cerr << "============================================\n";\
+    cerr << "|  A bad typeid     has been detected !!!  |\n";\
+    cerr << "============================================\n";\
+  } \
+  catch(std::bad_exception) { \
+    cerr << "============================================\n";\
+    cerr << "|  A bad exception  has been detected !!!  |\n";\
+    cerr << "============================================\n";\
+  } \
   catch(...) {\
     cerr << "============================================\n";\
     cerr << "|  An unknown error has been detected !!!  |\n";\
     cerr << "============================================\n";\
   }
+//   catch(ios_base::failure) {  a mettre plus tard
+//     cerr << "============================================\n";
+//     cerr << "|  A ios_base::failure has been detected !!!|\n";
+//     cerr << "============================================\n";
+//   } 
 
+#if (!defined(__GNUC__) || __GNUC__ < 3)
   #define DAL_THROW(type, thestr) { \
     std::strstream msg; \
     msg << "in "__FILE__ << ", line " << __LINE__ << ": \n" << thestr << ends;\
+    std::string s = msg.str();  /* copy the content to the string */ \
+    msg.freeze(false);/* and un-freeze the strstream to avoid the mem leak */ \
+    throw (type)(s); \
+  }
+#else
+  #define DAL_THROW(type, thestr) { \
+    std::stringstream msg; \
+    msg << "in "__FILE__ << ", line " << __LINE__ << ": \n" << thestr << ends;\
     throw (type)(msg.str()); \
   }
-
+#endif
 } /* end of namespace dal.                                                */
 
 
