@@ -55,10 +55,14 @@ namespace getfem
     size_type cv, nbd1, nbd2, N = mf.linked_mesh().dim();
     dal::bit_vector nn = mf.convex_index();
     base_tensor t;
-    pfem pf1, pf2, pf1prec = NULL, pf2prec = NULL;
-    pintegration_method pim, pimprec = (bgeot::papprox_integration)(NULL);
-    bgeot::pgeometric_trans pgt, pgtprec = NULL;
-    pmat_elem_computation pmec;
+    pfem pf1, pf2, pf1prec = 0, pf2prec = 0;
+    bgeot::pintegration_method pim, pimprec = 0;
+    bgeot::pgeometric_trans pgt, pgtprec = 0;
+    pmat_elem_computation pmec = 0;
+
+    if (&(mf.linked_mesh()) != &(mfdata.linked_mesh()))
+      DAL_THROW(std::invalid_argument,
+		"This assembling procedure only works on a single mesh");
 
     for (cv << nn; cv != ST_NIL; cv << nn)
     {
@@ -112,18 +116,19 @@ namespace getfem
 
     base_tensor t;
 
-    pmat_elem_computation pmec;
+    pmat_elem_computation pmec = 0;
 
     pfem pf_u, pf_p, pf_d;
     pfem pf_u_prec = NULL, pf_p_prec = NULL, pf_d_prec = NULL;
-    pintegration_method pim, pimprec = (bgeot::papprox_integration)(NULL);
+    bgeot::pintegration_method pim, pimprec = 0;
     bgeot::pgeometric_trans pgt, pgtprec = NULL;
     size_type nbdof_u, nbdof_p, nbdof_d;
-
     size_type N = mf_u.linked_mesh().dim();
-    if (N != mf_p.linked_mesh().dim() || N != mf_d.linked_mesh().dim())
-      throw dimension_error
-	("assembling_mixed_pressure_term : dimensions mismatch");
+
+    if (&(mf_u.linked_mesh()) != &(mf_p.linked_mesh())
+	|| &(mf_u.linked_mesh()) != &(mf_d.linked_mesh()))
+      DAL_THROW(std::invalid_argument,
+		"This assembling procedure only works on a single mesh");
 
     /* loop over all convexes */
     for (cv << nn; cv != ST_NIL; cv << nn) {
@@ -191,9 +196,13 @@ namespace getfem
     dal::bit_vector nn = mf.convex_index();
     base_tensor t;
     pfem pf1, pf2, pf1prec = NULL, pf2prec = NULL;
-    pintegration_method pim, pimprec = (bgeot::papprox_integration)(NULL);
+    bgeot::pintegration_method pim, pimprec = 0;
     bgeot::pgeometric_trans pgt, pgtprec = NULL;
-    pmat_elem_type pme; pmat_elem_computation pmec;
+    pmat_elem_type pme; pmat_elem_computation pmec = 0;
+
+    if (&(mf.linked_mesh()) != &(mfdata.linked_mesh()))
+      DAL_THROW(std::invalid_argument,
+		"This assembling procedure only works on a single mesh");
   
     for (cv << nn; cv != ST_NIL; cv << nn)
     {
@@ -278,9 +287,13 @@ namespace getfem
     dal::bit_vector nn = mf_u.convex_index(), nf;
     base_tensor t;
     pfem pf_u, pf_d, pf_u_prec = NULL, pf_d_prec = NULL;
-    pintegration_method pim, pimprec = (bgeot::papprox_integration)(NULL);
+    bgeot::pintegration_method pim, pimprec = 0;
     bgeot::pgeometric_trans pgt, pgtprec = NULL;
-    pmat_elem_type pme; pmat_elem_computation pmec;
+    pmat_elem_type pme; pmat_elem_computation pmec = 0;
+
+    if (&(mf_u.linked_mesh()) != &(mf_d.linked_mesh()))
+      DAL_THROW(std::invalid_argument,
+		"This assembling procedure only works on a single mesh");
 
     for (cv << nn; cv != ST_NIL; cv << nn)
     {
@@ -341,50 +354,59 @@ namespace getfem
   /* ********************************************************************* */
 
   template<class MATRM, class MESH_FEM>
-    void mass_matrix(MATRM &M, MESH_FEM &mf, dim_type N)
+    void mass_matrix(MATRM &M, MESH_FEM &mf1, MESH_FEM &mf2, dim_type N)
   {
-    size_type cv;
-    dal::bit_vector nn = mf.convex_index();
+    size_type cv, nbd1, nbd2;
+    dal::bit_vector nn = mf1.convex_index();
     base_tensor t;
-    pfem pf1, pf1prec = NULL;
-    pintegration_method pim, pimprec = (bgeot::papprox_integration)(NULL);
+    pfem pf1, pf1prec = 0, pf2, pf2prec = 0;
+    bgeot::pintegration_method pim, pimprec = 0;
     bgeot::pgeometric_trans pgt, pgtprec = NULL;
-    pmat_elem_type pme; pmat_elem_computation pmec;
+    pmat_elem_type pme; pmat_elem_computation pmec = 0;
     // M(0,0) = 1.0;  ??
 
+    if (&(mf1.linked_mesh()) != &(mf2.linked_mesh()))
+      DAL_THROW(std::invalid_argument,
+		"This assembling procedure only works on a single mesh");
+
     for (cv << nn; cv != ST_NIL; cv << nn)
     {
-      pf1 = mf.fem_of_element(cv);
-      pgt = mf.linked_mesh().trans_of_convex(cv);
-      pim = mf.int_method_of_element(cv);
-      if (pf1prec != pf1 || pgtprec != pgt || pimprec != pim)
+      pf1 = mf1.fem_of_element(cv); nbd1 = pf1->nb_dof();
+      pf2 = mf2.fem_of_element(cv); nbd2 = pf2->nb_dof();
+      pgt = mf1.linked_mesh().trans_of_convex(cv);
+      pim = mf1.int_method_of_element(cv);
+      if (pf1prec != pf1 || pf2prec != pf2 || pgtprec != pgt || pimprec != pim)
       {
-	pme = mat_elem_product(mat_elem_base(pf1), mat_elem_base(pf1));
+	pme = mat_elem_product(mat_elem_base(pf1), mat_elem_base(pf2));
 	pmec = mat_elem(pme, pim, pgt);
-	pf1prec = pf1; pgtprec = pgt; pimprec = pim;
+	pf1prec = pf1; pf2prec = pf2; pgtprec = pgt; pimprec = pim;
       }
-      pmec->gen_compute(t, mf.linked_mesh().points_of_convex(cv));
+      pmec->gen_compute(t, mf1.linked_mesh().points_of_convex(cv));
 
+      // cout << "t = " << t << endl;
+      
       base_tensor::iterator p = t.begin();
-      size_type nbd = mf.nb_dof_of_element(cv);
-      for (size_type i = 0; i < nbd; i++)
+      for (size_type i = 0; i < nbd2; i++)
       {
-	size_type dof1 = mf.ind_dof_of_element(cv)[i];
-	for (size_type j = 0; j < nbd; j++, ++p)
+	size_type dof2 = mf2.ind_dof_of_element(cv)[i];
+	// cout << "cv = " << cv << " dof2 = " << dof2 << endl;
+	for (size_type j = 0; j < nbd1; j++, ++p)
 	{
-	  size_type dof2 = mf.ind_dof_of_element(cv)[j];
-	  if (dof2 <= dof1)
-	    for (size_type k = 0; k < N; k++)
-	    {
-	      M(dof1*N + k, dof2*N + k) += (*p);
-	      M(dof2*N + k, dof1*N + k) = M(dof1*N + k, dof2*N + k);
-	    }
+	  size_type dof1 = mf1.ind_dof_of_element(cv)[j];
+	  // cout << "dof1 = " << dof1 << " dof2 = " << dof2 << endl;
+	  for (size_type k = 0; k < N; k++)
+	    M(dof1*N + k, dof2*N + k) += (*p);
 	}
       }
       if (p != t.end()) DAL_THROW(dal::internal_error, "internal error"); 
     }
   }
 
+  template<class MATRM, class MESH_FEM>
+    inline void mass_matrix(MATRM &M, MESH_FEM &mf, dim_type N)
+    { mass_matrix(M, mf, mf, N); }
+
+
   /* ********************************************************************* */
   /*	volumic source term.                                               */
   /* ********************************************************************* */
@@ -397,9 +419,14 @@ namespace getfem
     dal::bit_vector nn = mf.convex_index();
     base_tensor t;
     pfem pf1, pf2, pf1prec = NULL, pf2prec = NULL;
-    pintegration_method pim, pimprec = (bgeot::papprox_integration)(NULL);
+    bgeot::pintegration_method pim, pimprec = 0;
     bgeot::pgeometric_trans pgt, pgtprec = NULL;
-    pmat_elem_type pme; pmat_elem_computation pmec;
+    pmat_elem_type pme; pmat_elem_computation pmec = 0;
+
+    if (&(mf.linked_mesh()) != &(mfdata.linked_mesh()))
+      DAL_THROW(std::invalid_argument,
+		"This assembling procedure only works on a single mesh");
+
     for (cv << nn; cv != ST_NIL; cv << nn)
     {
       pf1 =     mf.fem_of_element(cv); nbd1 = pf1->nb_dof();
@@ -435,25 +462,31 @@ namespace getfem
     void assembling_Dirichlet_condition(MATRM &RM, VECT1 &B, mesh_fem &mf,
 		  size_type boundary, VECT2 &F, dim_type N)
   { /* Y-a-il un moyen plus performant ? */
+    // Marche uniquement pour des ddl de lagrange.
     size_type cv;
     dal::bit_vector nn = mf.convex_index();
     dal::bit_vector nndof = mf.dof_on_boundary(boundary);
+    pfem pf1;
 
     for (cv << nn; cv != ST_NIL; cv << nn)
     {
-      size_type nbd = mf.nb_dof_of_element(cv);
+      pf1 = mf.fem_of_element(cv);
+      pdof_description ldof = lagrange_dof(pf1->dim());
+      size_type nbd = pf1->nb_dof();
       for (size_type i = 0; i < nbd; i++)
       {
 	size_type dof1 = mf.ind_dof_of_element(cv)[i];
-	if (nndof.is_in(dof1))
+	if (nndof.is_in(dof1) && pf1->dof_types()[i] == ldof)
 	{
+	  // cout << "dof : " << i << endl;
 	  for (size_type j = 0; j < nbd; j++)
 	  {
 	    size_type dof2 = mf.ind_dof_of_element(cv)[j];
 	    for (size_type k = 0; k < N; ++k)
 	      for (size_type l = 0; l < N; ++l)
 	      {
-		if (!(nndof.is_in(dof2)))
+		if (!(nndof.is_in(dof2)) 
+		    && pf1->dof_types()[j] == lagrange_dof(pf1->dim()))
 		  B[dof2*N+k] -= RM(dof2*N+k, dof1*N+l) * F[dof1*N+l];
 		RM(dof2*N+k, dof1*N+l) = RM(dof1*N+l, dof2*N+k) = 0;
 	      }
@@ -485,6 +518,7 @@ namespace getfem
       for (i = j = k = 0; i < s; ++i)
 	if (nn[i]) G(i,j++) = 1.0; else B(i, k++) = 1.0;
       
+      // ...
       // il faut rsoudre le systme B^T D B UDDR = B^T UD
       // On a alors UDD = B UDDR
       // pour cela faire une factorisation LU (cf MATLAB null ...)
@@ -507,9 +541,13 @@ namespace getfem
     dal::bit_vector nn = mf.convex_index(), nf;
     base_tensor t;
     pfem pf1, pf2, pf1prec = NULL, pf2prec = NULL;
-    pintegration_method pim, pimprec = (bgeot::papprox_integration)(NULL);
+    bgeot::pintegration_method pim, pimprec = 0;
     bgeot::pgeometric_trans pgt, pgtprec = NULL;
-    pmat_elem_type pme; pmat_elem_computation pmec;
+    pmat_elem_type pme; pmat_elem_computation pmec = 0;
+
+    if (&(mf.linked_mesh()) != &(mfdata.linked_mesh()))
+      DAL_THROW(std::invalid_argument,
+		"This assembling procedure only works on a single mesh");
   
     for (cv << nn; cv != ST_NIL; cv << nn)
     {
