@@ -49,7 +49,7 @@ namespace getfem
     size_type N = mf.linked_mesh().dim();
     
     if (&mf.linked_mesh() != &mf_target.linked_mesh())
-      DAL_THROW(std::invalid_argument, "meshes are not the same");
+      DAL_THROW(std::invalid_argument, "meshes are different.");
 
     base_matrix G, val;
     base_vector coeff;
@@ -67,6 +67,8 @@ namespace getfem
       if (!(pf_target->is_equivalent()) || !(pf_target->is_lagrange()))
 	DAL_THROW(std::invalid_argument, 
 		  "finite element target not convenient");
+      if (!(pf->is_equivalent())) 
+	transfert_to_G(G, mf.linked_mesh().points_of_convex(cv));
 
       pgt = mf.linked_mesh().trans_of_convex(cv);
       if (pf_targetold != pf_target) {
@@ -81,11 +83,10 @@ namespace getfem
 
       size_type P = pgt->structure()->dim(); /* dimension of the convex.*/
       base_matrix a(N, pgt->nb_points());
-      base_matrix pc(pgt->nb_points() , P);
       base_matrix grad(N, P), TMP1(P,P), B0(P,N), B1(1, N), CS(P,P);
-      size_type nbpt = 0, i;
       
       /* TODO: prendre des iterateurs pour faire la copie */
+      // utiliser transfert_to_G ?
       for (size_type j = 0; j < pgt->nb_points(); ++j) //  optimiser !!
 	for (size_type i = 0; i < N; ++i)
 	  a(i,j) = mf.linked_mesh().points_of_convex(cv)[j][i];
@@ -114,7 +115,7 @@ namespace getfem
 	for (size_type q = 0; q < Q; ++q) {
 	  for (size_type l = 0; l < pf->nb_dof(); ++l)
 	    coeff[l] = U[mf.ind_dof_of_element(cv)[l] * Q + q ];
-	  pf->interpolation_grad(pfp, j, G, coeff, val);
+	  pf->interpolation_grad(pfp, j, G, pgt, coeff, val);
 	  bgeot::mat_product(val, B0, B1);
 
 	  for (size_type l = 0; l < N; ++l)
