@@ -33,6 +33,7 @@
 #define __GETFEM_EXPORT_H
 
 #include <getfem_mesh_fem.h>
+#include <getfem_mat_elem.h>
 #include <bgeot_geotrans_inv.h>
 #include <dal_tree_sorted.h>
 
@@ -50,7 +51,7 @@ namespace getfem
 		       const VECT &U, dim_type P, short_type K)
   { 
     dim_type N = mf.linked_mesh().dim();
-    std::ofstream o((filename + char(0)).data());
+    std::ofstream o(filename.c_str());
     if (!o) DAL_THROW(internal_error, "impossible to open file");
     dal::bit_vector nn = mf.convex_index();
     size_type cv;
@@ -68,29 +69,26 @@ namespace getfem
       pfem pf1 = mf.fem_of_element(cv);
       size_type nbd1 = mf.nb_dof_of_element(cv);
       size_type nbd2 = pfe->nb_dof();
-      size_type nbpt = pgt->nb_points();
       coeff.resize(nbd1);
 
-      o << "DIM = " << pgt->dim() << endl;
+      o << "DIM = " << int(pgt->dim()) << endl;
 
-      if (pf1->target_dim() != 1 || !(pf1->is_equivalent()))
+      if (!(pf1->is_equivalent())) 
+	transfert_to_G(G, mf.linked_mesh().points_of_convex(cv));
+
+      if (pf1->target_dim() != 1)
 	DAL_THROW(to_be_done_error, "to be done ... ");
       
       for (size_type i = 0; i < nbd2; ++i)
       {
 	/* point corresponding to the i th node.                           */
-	const base_node *ppt = &(pfe->node_of_dof(i));
-	pt1.fill(0.0);
-	for (size_type j = 0; j < nbpt; ++j)
-	{
-	  pt1.addmul(pgt->poly_vector()[j].eval(ppt->begin()),
-		   mf.linked_mesh().points_of_convex(cv)[j]);
-	}
+	pt1 = pgt->transform(pfe->node_of_dof(i),
+			     mf.linked_mesh().points_of_convex(cv));
 	for (size_type j = 0; j < N; ++j) o << pt1[j] << " ";
 	o << "  ";
 
 	/* interpolation of the solution.                                  */
-	/* faux dans le cas des lments non tau-equivalents ou vectoriel. */
+	/* faux dans le cas des lments vectoriel.                        */
 	pt2 = pfe->node_of_dof(i);
 	for (size_type k = 0; k < P; ++k) {
 	  for (size_type j = 0; j < nbd1; ++j) {
@@ -98,7 +96,7 @@ namespace getfem
 	    coeff[j] = U[dof1*P+k];
 	  }
 	  // il faudrait utiliser les fem_precomp pour accelerer.
-	  pf1->interpolation(pt2, G, coeff, val);
+	  pf1->interpolation(pt2, G, pgt, coeff, val);
 	  pt3[k] = val[0];
 	}
 
@@ -114,7 +112,7 @@ namespace getfem
   template<class VECT>
     void load_solution(const std::string &fi, getfem_mesh &mesh,
 		       mesh_fem &mef, VECT &U, dim_type &P, short_type &K) {
-    std::ifstream ist((fi + char(0)).data());
+    std::ifstream ist(fi.c_str());
     size_type token, DIM, Np, N;
     char tmp[100], c;
     bool error = false, began = false;
@@ -163,9 +161,9 @@ namespace getfem
 	//      << K << " DIM = " << DIM << endl;
 	while(nbpt < 1000) {
 	  if (ptab[nbpt].size() != N) ptab[nbpt].resize(N);
-	  for (int i = 0; i < N; ++i) ist >> (ptab[nbpt])[i];
+	  for (dim_type i = 0; i < N; ++i) ist >> (ptab[nbpt])[i];
 	  // cout << "Ptab[" << nbpt << "] = " << ptab[nbpt] << endl;
-	  for (int i = 0; i < P; ++i) ist >> vtab[nbvtab * P + i];
+	  for (short_type i = 0; i < P; ++i) ist >> vtab[nbvtab * P + i];
 	  nbpt++; nbvtab++;
 	  while(ist.get(c))
 	    { if (!(isspace(c)) || c == 10) { ist.putback(c); break; } }
@@ -177,20 +175,20 @@ namespace getfem
 	if (nbpt >= 1000) { error = true; break; }
 	if (nbpt == 0) break;
 
-	Np = 1; for (int i = 0; i < DIM; ++i) Np *= K + 1;
+	Np = 1; for (dim_type i = 0; i < DIM; ++i) Np *= K + 1;
 	
 	if (nbpt == bgeot::alpha(DIM, K)) {
 	  std::vector<getfem::base_node> pnode(DIM+1);
-	  for (int i = 0; i <= DIM; ++i) 
+	  for (dim_type i = 0; i <= DIM; ++i) 
 	    pnode[i] = ptab[bgeot::alpha(i, K) - 1];
-	  int i = mesh.add_simplex_by_points(DIM, pnode.begin());
+	  size_type i = mesh.add_simplex_by_points(DIM, pnode.begin());
 	  mef.set_finite_element(i, getfem::PK_fem(DIM, K),
-				 bgeot::simplex_poly_integration(DIM));
+				 bgeot::exact_simplex_im(DIM));
 	}
 	else if (nbpt == Np) {
 	  std::vector<getfem::base_node> pnode(1 << DIM);
 	  size_type j = 0;
-	  for (int i = 0; i <= (1 << DIM); ++i) {
+	  for (size_type i = 0; i <= (size_type(1) << DIM); ++i) {
 	    pnode[i] = ptab[j];
 	    size_type k = i + 1, l = K-1;
 	    while (!(k & 1)) { l *= K+1; k >>= 1; }
@@ -198,17 +196,17 @@ namespace getfem
 	  }
 	  j = mesh.add_parallelepiped_by_points(DIM, pnode.begin());
 	  mef.set_finite_element(j, getfem::QK_fem(DIM, K),
-				 bgeot::parallelepiped_poly_integration(DIM));
+				 bgeot::exact_parallelepiped_im(DIM));
 	}
 	else if (nbpt == bgeot::alpha(DIM - 1, K) * bgeot::alpha(1, K)) {
 	  std::vector<getfem::base_node> pnode(2*DIM);
-	  for (int i = 0; i < DIM; ++i)
+	  for (dim_type i = 0; i < DIM; ++i)
 	    pnode[i] = ptab[bgeot::alpha(i, K) - 1];
-	  for (int i = 0; i < DIM; ++i)
+	  for (dim_type i = 0; i < DIM; ++i)
 	    pnode[i+DIM] = ptab[bgeot::alpha(i, K)-1 + bgeot::alpha(DIM-1, K)];
 	  int i = mesh.add_prism_by_points(DIM, pnode.begin());
 	  mef.set_finite_element(i, getfem::PK_prism_fem(DIM, K),
-				 bgeot::prism_poly_integration(DIM));
+				 bgeot::exact_prism_im(DIM));
 	}
 	else 
 	  DAL_THROW(failure_error, "Unknown element in file " << fi);
@@ -228,15 +226,16 @@ namespace getfem
     size_type l = 0, i;
     for (i << nn; i != size_type(-1); i << nn) {
       size_type nbd = mef.nb_dof_of_element(i);
-      for (int j = 0; j < nbd; ++j, ++l) {
+      for (size_type j = 0; j < nbd; ++j, ++l) {
 	size_type dof = mef.ind_dof_of_element(i)[j];
-	for (int k = 0; k < P; ++k) U[dof*P +k] += vtab[l*P+k];
+	for (short_type k = 0; k < P; ++k) U[dof*P +k] += vtab[l*P+k];
 	(cp[dof])++;
       }
     }
     for (i = 0; i < mef.nb_dof(); ++i) {
       if (cp[i] == 0) DAL_THROW(internal_error, "Internal error");
-      for (int k = 0; k < P; ++k) U[i*P +k] /= getfem::scalar_type(cp[i]);
+      for (short_type k = 0; k < P; ++k)
+	U[i*P +k] /= getfem::scalar_type(cp[i]);
     }
     
   }
@@ -247,15 +246,16 @@ namespace getfem
   /*                                                                       */
   /* ********************************************************************* */
 
-  struct edge_list_elt
-  {
+  struct edge_list_elt  {
     size_type i, j;
+    size_type cv;
     inline bool operator < (const edge_list_elt &e) const
     {
       if (i < e.i) return true; if (i > e.i) return false; 
-      if (j < e.j) return true; return false;
+      if (j < e.j) return true; else if (j > e.j) return false;
+      if (cv < e.cv) return true; return false;
     }
-    edge_list_elt(size_type ii, size_type jj)
+    edge_list_elt(size_type ii, size_type jj, size_type ic = 0) : cv(ic)
     { i = std::min(ii, jj); j = std::max(ii, jj); }
     edge_list_elt(void) {}
   };
@@ -263,7 +263,10 @@ namespace getfem
   typedef dal::dynamic_tree_sorted<edge_list_elt> edge_list;
   
 
-  void mesh_edges_list(const getfem_mesh &m, edge_list &el);
+  void mesh_edge_list_convex(const getfem_mesh &m, size_type i, 
+			     edge_list &el, bool merge_convex = true);
+  void mesh_edges_list(const getfem_mesh &m, edge_list &el, 
+		       bool merge_convex = true);
 
 
   /* ********************************************************************* */
@@ -278,7 +281,6 @@ namespace getfem
     void interpolation_solution(mesh_fem &mf, mesh_fem &mf_target,
 				const VECT &U, VECT &V, dim_type P)
   {
-    dim_type N = mf.linked_mesh().dim();
     size_type cv, nb;
     base_node pt3(P), val(1);
     bgeot::geotrans_inv gti;
@@ -292,18 +294,23 @@ namespace getfem
     nb = mf_target.nb_dof();
     for (size_type i = 0; i < nb; ++i)
       gti.add_point(mf_target.point_of_dof(i));
+    // il faudrait controler que tous les ddl de mf_target sont de
+    // type lagrange
 
     dal::bit_vector nn = mf.convex_index(), ddl_touched;
     ddl_touched.add(0, nb-1);
 
     for (cv << nn; cv != ST_NIL; cv << nn)
     {
+      bgeot::pgeometric_trans pgt = mf.linked_mesh().trans_of_convex(cv);
       // cout << "dealing with convex " << cv << endl;
       nb = gti.points_in_convex(mf.linked_mesh().convex(cv),
 			    mf.linked_mesh().trans_of_convex(cv), ptab, itab);
       // cout << "nb points in this convex " << nb << endl;
       // cout << "convex : " << mf.linked_mesh().convex(cv) << endl;
       pfem pfe = mf.fem_of_element(cv);
+      if (!(pfe->is_equivalent())) 
+	transfert_to_G(G, mf.linked_mesh().points_of_convex(cv));
       size_type nbd1 = pfe->nb_dof();
       coeff.resize(nbd1);
       for (size_type i = 0; i < nb; ++i)
@@ -318,9 +325,8 @@ namespace getfem
 	      size_type dof1 = mf.ind_dof_of_element(cv)[j];
 	      coeff[j] = U[dof1*P+k];
 	    }
-	    pfe->interpolation(ptab[i], G, coeff, val);
+	    pfe->interpolation(ptab[i], G, pgt, coeff, val);
 	    pt3[k] = val[0];
-
 	  }
 	  
 	  // cout << "Rsultat : " << pt3 << endl;
@@ -336,6 +342,60 @@ namespace getfem
   }
 
 
+  /* ********************************************************************* */
+  /*                                                                       */
+  /*  interpolation of a solution on same mesh.                            */
+  /*   - mf_target must be of lagrange type.                               */
+  /*                                                                       */
+  /* ********************************************************************* */
+
+  template<class VECT>
+    void interpolation_solution_same_mesh(mesh_fem &mf, mesh_fem &mf_target,
+					  const VECT &U, VECT &V, dim_type P)
+  {
+    dal::bit_vector nn = mf.convex_index();
+    size_type cv;
+    base_node pt2, val(1);
+    base_matrix G;
+    base_vector coeff;
+
+    if ( &(mf.linked_mesh()) != &(mf_target.linked_mesh()))
+      DAL_THROW(failure_error, "Meshes should be the same in this function.");
+
+    for (cv << nn; cv != ST_NIL; cv << nn)
+    {
+      bgeot::pgeometric_trans pgt = mf.linked_mesh().trans_of_convex(cv);
+      pfem pfe = mf_target.fem_of_element(cv);
+      pfem pf1 = mf.fem_of_element(cv);
+      size_type nbd1 = mf.nb_dof_of_element(cv);
+      size_type nbd2 = pfe->nb_dof();
+      coeff.resize(nbd1);
+
+      if (!(pf1->is_equivalent())) 
+	transfert_to_G(G, mf.linked_mesh().points_of_convex(cv));
+
+      if (pf1->target_dim() != 1)
+	DAL_THROW(to_be_done_error, "to be done ... ");
+      
+      for (size_type i = 0; i < nbd2; ++i) {
+	size_type dof1 = mf_target.ind_dof_of_element(cv)[i];
+	/* interpolation of the solution.                                  */
+	/* faux dans le cas des lments vectoriel.                        */
+	pt2 = pfe->node_of_dof(i);
+	for (size_type k = 0; k < P; ++k) {
+	  for (size_type j = 0; j < nbd1; ++j) {
+	    size_type dof1 = mf.ind_dof_of_element(cv)[j];
+	    coeff[j] = U[dof1*P+k];
+	  }
+	  // il faudrait utiliser les fem_precomp pour accelerer.
+	  pf1->interpolation(pt2, G, pgt, coeff, val);
+	  V[dof1*P + k] = val[0];
+	}
+      }
+    }
+  }
+
+
   
 }  /* end of namespace getfem.                                             */
 
