@@ -47,10 +47,10 @@ namespace getfem
     dal::dynamic_array<base_vector, 2> vval;
     base_tensor t;
     pfem pf1, pf1prec = NULL;
-    pintegration_method pim, pimprec = (bgeot::papprox_integration)(NULL);
+    bgeot::pintegration_method pim, pimprec = 0;
 
     bgeot::pgeometric_trans pgt, pgtprec = NULL;
-    pmat_elem_type pme; pmat_elem_computation pmec;
+    pmat_elem_type pme; pmat_elem_computation pmec = 0;
     
     for (cv << nn; cv != ST_NIL; cv << nn)
     {
@@ -91,9 +91,9 @@ namespace getfem
     dal::dynamic_array<base_vector, 2> vval;
     base_tensor t;
     pfem pf1, pf1prec = NULL;
-    pintegration_method pim, pimprec = (bgeot::papprox_integration)(NULL);
+    bgeot::pintegration_method pim, pimprec = 0;
     bgeot::pgeometric_trans pgt, pgtprec = NULL;
-    pmat_elem_type pme; pmat_elem_computation pmec;
+    pmat_elem_type pme; pmat_elem_computation pmec = 0;
 
     for (cv << nn; cv != ST_NIL; cv << nn)
     {
